interface PropertyCondition {
  is: string | number | boolean;
}

type Callback = () => void;

export function Functional<T extends Record<string, any>>(data: T) {
  /** Flag global para saber se algum .property() disparou */
  let matched = false;

  /** Objeto que vamos devolver em todas as chamadas para manter o encadeamento */
  const chain = {
    /* --------------------------------------------------------- */
    /* .property('key', { is }, cb)                              */
    /* --------------------------------------------------------- */
    property(key: keyof T, directive: PropertyCondition, cb: Callback) {
      if (data[key] === directive.is) {
        matched = true;
        cb();
      }
      return chain; // permite .property().property().continue()
    },

    /* --------------------------------------------------------- */
    /* .continue(cb) – executa cb se NENHUM .property() bateu    */
    /* --------------------------------------------------------- */
    continue(cb: Callback) {
      if (!matched) cb();
      matched = false; // zera para um novo ciclo, se encadear
      return chain;
    },

    /* --------------------------------------------------------- */
    /* .some(directive).then(cb).catch(cb)                       */
    /* --------------------------------------------------------- */
    some<Dir>(directive: Dir) {
      let isTrue = false;
      if (Array.isArray(data)) {
        isTrue = (data as unknown[]).some((item) => item === directive);
      }

      return {
        then(cb: Callback) {
          if (isTrue) cb();
          return this;
        },
        catch(cb: Callback) {
          if (!isTrue) cb();
          return this;
        },
      };
    },
  };

  return chain;
}
