import { transformSync } from '@babel/core';
import { readFileSync, writeFileSync } from 'node:fs';
import path from 'node:path';
import plugin from '..';
import ts from 'typescript';

const config = {
  filename: 'styles.js',
  // Preset env cobre recursos ES7
  presets: [
    [
      '@babel/preset-env',
      {
        targets: {
          chrome: '97',
          firefox: '96',
          safari: '15.4',
          node: '16.14',
        },
        modules: false,
        useBuiltIns: false,
        bugfixes: false,
      },
    ],
  ],
  // Plugins específicos do ES7 que você talvez queira incluir
  plugins: [plugin],
};

const tsCode = readFileSync(path.join(__dirname, 'styles.ts'), 'utf-8');

const jsCode = ts.transpileModule(tsCode, {
  compilerOptions: {
    module: ts.ModuleKind.ESNext,
    target: ts.ScriptTarget.ESNext,
    importHelpers: false,
    noEmitHelpers: true,
  },
});

const result = transformSync(jsCode.outputText, {
  ...config,
  sourceType: 'module',
});

writeFileSync(path.join(__dirname, 'styles.js'), result?.code || '');
